@@ -6,11 +6,11 @@
  * F. Durand and J. Dorsey.
  * In ACM Transactions on Graphics, 2002.
  *
- * 
+ *
  * This file is a part of LuminanceHDR package, based on pfstmo.
- * ---------------------------------------------------------------------- 
+ * ----------------------------------------------------------------------
  * Copyright (C) 2003,2004 Grzegorz Krawczyk
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -24,8 +24,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * ---------------------------------------------------------------------- 
- * 
+ * ----------------------------------------------------------------------
+ *
  * @author Grzegorz Krawczyk, <krawczyk@mpi-sb.mpg.de>
  *
  * $Id: pfstmo_durand02.cpp,v 1.5 2009/02/23 19:09:41 rafm Exp $
@@ -38,13 +38,9 @@
 #include "Libpfs/colorspace/colorspace.h"
 #include "Libpfs/exception.h"
 #include "Libpfs/frame.h"
+#include "Libpfs/progress.h"
 #include "tmo_durand02.h"
 
-namespace pfs
-{
-class Progress;
-}
-
 namespace
 {
 const int downsample = 1;
@@ -63,7 +59,7 @@ const bool original_algorithm = false;
 void pfstmo_durand02(pfs::Frame& frame,
                      float sigma_s, float sigma_r, float baseContrast,
                      pfs::Progress &ph)
-{ 
+{
 #ifndef NDEBUG
     std::stringstream ss;
 
@@ -81,7 +77,7 @@ void pfstmo_durand02(pfs::Frame& frame,
 #endif
 
   pfs::Channel *X, *Y, *Z;
-  
+
   frame.getXYZChannels( X, Y, Z );
   frame.getTags().setTag("LUMINANCE", "RELATIVE");
   //---
@@ -90,9 +86,12 @@ void pfstmo_durand02(pfs::Frame& frame,
   {
     throw pfs::Exception( "Missing X, Y, Z channels in the PFS stream" );
   }
-  
+
   tmo_durand02(*X, *Y, *Z,
                sigma_s, sigma_r, baseContrast, downsample, !original_algorithm,
                ph);
+
+  if ( !ph.canceled() )
+      ph.setValue(100);
 }
 
