@@ -29,42 +29,50 @@
 #include <QtConcurrentMap>
 #include <QtConcurrentFilter>
 #include <QDebug>
+#include <QRgb>
 #include <QImage>
 #include <QPixmap>
+#include <QRgb>
 
-#include <valarray> 
-
-#include <boost/scoped_ptr.hpp>
-#include <boost/foreach.hpp>
 #include <boost/bind.hpp>
 
 #include "FitsImporter.h"
 #include "ui_FitsImporter.h"
 
 #include "Common/CommonFunctions.h"
-#include "Libpfs/manip/rotate.h"
-#include <Libpfs/utils/transform.h>
 #include "HdrCreation/mtb_alignment.h"
 
+#include <UI/UMessageBox.h>
+#include <Libpfs/manip/rotate.h>
+#include <Libpfs/utils/transform.h>
+#include <Libpfs/colorspace/convert.h>
+#include <Libpfs/colorspace/normalizer.h>
+
 using namespace pfs;
+using namespace pfs::colorspace;
 
 static const int previewWidth = 300;
 static const int previewHeight = 200;
 
-FitsImporter::~FitsImporter() 
+FitsImporter::~FitsImporter()
 {
-    if (m_align)
-        delete m_align;
 }
 
 FitsImporter::FitsImporter(QWidget *parent)
     : QWizard(parent)
     , m_width(0)
     , m_height(0)
-    , m_align(NULL)
-    , m_ui(new Ui::FitsImporter)
+    , m_align()
+    , m_Ui(new Ui::FitsImporter)
 {
-    m_ui->setupUi(this);
+    m_Ui->setupUi(this);
+
+    if ( !QIcon::hasThemeIcon("upload-media") )
+        m_Ui->pushButtonLoad->setIcon(QIcon(":/program-icons/upload-media"));
+    if ( !QIcon::hasThemeIcon("chronometer-reset") )
+        m_Ui->pushButtonReset->setIcon(QIcon(":/program-icons/chronometer-reset"));
+    if ( !QIcon::hasThemeIcon("document-preview") )
+        m_Ui->pushButtonPreview->setIcon(QIcon(":/program-icons/document-preview"));
 
 #ifdef WIN32
     setWizardStyle(WizardStyle::ModernStyle);
@@ -73,20 +81,20 @@ FitsImporter::FitsImporter(QWidget *parent)
     this->setWindowModality(Qt::WindowModal); // In OS X, the QMessageBox is modal to the window
 #endif
 
-    setPixmap(WizardPixmap::LogoPixmap, QIcon(":/new/prefix1/images/Galaxy-icon.png").pixmap(48, 48));
+    setPixmap(WizardPixmap::LogoPixmap, QIcon(":/program-icons/Galaxy-icon").pixmap(48, 48));
 
     m_previewLabel = new QLabel(this);
     m_previewLabel->resize(600,400);
     m_previewLabel->setScaledContents(true);
-    QPalette* palette = new QPalette(); 
+    QPalette* palette = new QPalette();
     palette->setColor(QPalette::Foreground,Qt::red);
     m_previewLabel->setPalette(*palette);
     m_previewLabel->setFrameStyle(QFrame::Box);
     m_previewLabel->setLineWidth(3);
     m_previewLabel->hide();
-    m_ui->previewLabel->setPalette(*palette);
+    m_Ui->previewLabel->setPalette(*palette);
     m_previewFrame = new PreviewFrame;
-    m_ui->verticalLayoutPreviews->addWidget(m_previewFrame);
+    m_Ui->verticalLayoutPreviews->addWidget(m_previewFrame);
 
     for (int i = 0; i < 5; i++)
     {
@@ -96,18 +104,18 @@ FitsImporter::FitsImporter(QWidget *parent)
         connect(label, SIGNAL(selected(int)), this, SLOT(previewLabelSelected(int)));
     }
     m_previewFrame->show();
-    connect(this, SIGNAL(setValue(int)), m_ui->progressBar, SLOT(setValue(int)), Qt::DirectConnection);
-    connect(this, SIGNAL(setRange(int,int)), m_ui->progressBar, SLOT(setRange(int,int)), Qt::DirectConnection);
-    
-    // wizard stuff
-    
-    m_ui->wizardPageLoadFiles->setCompleteStatus(ExtWizardPage::CompleteStatus::AlwaysFalse);
-    m_ui->wizardPageLoadFiles->registerExtField("lineEditRed*", m_ui->lineEditRed);
-    m_ui->wizardPageLoadFiles->registerExtField("lineEditGreen*", m_ui->lineEditGreen);
-    m_ui->wizardPageLoadFiles->registerExtField("lineEditBlue*", m_ui->lineEditBlue);
+    connect(this, SIGNAL(setValue(int)), m_Ui->progressBar, SLOT(setValue(int)), Qt::DirectConnection);
+    connect(this, SIGNAL(setRange(int,int)), m_Ui->progressBar, SLOT(setRange(int,int)), Qt::DirectConnection);
 
-    m_ui->wizardPagePreview->setCommitPage(true);
-    m_ui->wizardPageAlignment->setCompleteStatus(ExtWizardPage::CompleteStatus::AlwaysFalse);
+    // wizard stuff
+
+    m_Ui->wizardPageLoadFiles->setCompleteStatus(ExtWizardPage::CompleteStatus::AlwaysFalse);
+    m_Ui->wizardPageLoadFiles->registerExtField("lineEditRed*", m_Ui->lineEditRed);
+    m_Ui->wizardPageLoadFiles->registerExtField("lineEditGreen*", m_Ui->lineEditGreen);
+    m_Ui->wizardPageLoadFiles->registerExtField("lineEditBlue*", m_Ui->lineEditBlue);
+
+    m_Ui->wizardPagePreview->setCommitPage(true);
+    m_Ui->wizardPageAlignment->setCompleteStatus(ExtWizardPage::CompleteStatus::AlwaysFalse);
 }
 
 pfs::Frame* FitsImporter::getFrame()
@@ -118,10 +126,10 @@ pfs::Frame* FitsImporter::getFrame()
 
 void FitsImporter::selectInputFile(QLineEdit* textField, QString* channel)
 {
-    QString filetypes = "FITS (*.fit *.FIT *.fits *.FITS);;";
+    QString filetypes = "FITS (*.fit *.FIT *.fits *.FITS)";
     *channel = QFileDialog::getOpenFileName(this, tr("Load one FITS image..."),
                                                       m_luminance_options.getDefaultPathHdrIn(),
-                                                      filetypes );
+                                                      filetypes);
     textField->setText(*channel);
     checkLoadButton();
 
@@ -140,32 +148,32 @@ void FitsImporter::selectInputFile(QLineEdit* textField, QString* channel)
 
 void FitsImporter::on_pushButtonLuminosity_clicked()
 {
-    selectInputFile(m_ui->lineEditLuminosity, &m_luminosityChannel);
+    selectInputFile(m_Ui->lineEditLuminosity, &m_luminosityChannel);
 }
 
 void FitsImporter::on_pushButtonRed_clicked()
 {
-    selectInputFile(m_ui->lineEditRed, &m_redChannel);
+    selectInputFile(m_Ui->lineEditRed, &m_redChannel);
 }
 
 void FitsImporter::on_pushButtonGreen_clicked()
 {
-    selectInputFile(m_ui->lineEditGreen, &m_greenChannel);
+    selectInputFile(m_Ui->lineEditGreen, &m_greenChannel);
 }
 
 void FitsImporter::on_pushButtonBlue_clicked()
 {
-    selectInputFile(m_ui->lineEditBlue, &m_blueChannel);
+    selectInputFile(m_Ui->lineEditBlue, &m_blueChannel);
 }
 
 void FitsImporter::on_pushButtonH_clicked()
 {
-    selectInputFile(m_ui->lineEditH, &m_hChannel);
+    selectInputFile(m_Ui->lineEditH, &m_hChannel);
 }
 
 void FitsImporter::checkLoadButton()
 {
-    m_ui->pushButtonLoad->setEnabled(!m_redChannel.isEmpty() ||
+    m_Ui->pushButtonLoad->setEnabled(!m_redChannel.isEmpty() ||
                                      !m_greenChannel.isEmpty() ||
                                      !m_blueChannel.isEmpty() ||
                                      !m_luminosityChannel.isEmpty() ||
@@ -174,55 +182,69 @@ void FitsImporter::checkLoadButton()
 
 void FitsImporter::on_pushButtonLoad_clicked()
 {
-    m_ui->wizardPageLoadFiles->setCompleteStatus(ExtWizardPage::CompleteStatus::AlwaysFalse);
+    m_Ui->wizardPageLoadFiles->setCompleteStatus(ExtWizardPage::CompleteStatus::AlwaysFalse);
 
     QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
-    m_ui->pushButtonLoad->setEnabled(false);
+    m_Ui->pushButtonLoad->setEnabled(false);
 
-    m_channels << m_redChannel
-               << m_greenChannel
-               << m_blueChannel
-               << m_luminosityChannel
-               << m_hChannel;
+    m_tmpdata.clear();
 
-    BOOST_FOREACH(const QString& i, m_channels) {
-        m_tmpdata.push_back( HdrCreationItem(i) );
-    }
+    m_tmpdata.push_back( HdrCreationItem(m_redChannel) );
+    m_tmpdata.push_back( HdrCreationItem(m_greenChannel) );
+    m_tmpdata.push_back( HdrCreationItem(m_blueChannel) );
+    m_tmpdata.push_back( HdrCreationItem(m_luminosityChannel) );
+    m_tmpdata.push_back( HdrCreationItem(m_hChannel) );
 
     // parallel load of the data...
-    connect(&m_futureWatcher, SIGNAL(finished()), this, SLOT(loadFilesDone()), Qt::DirectConnection);
+    // connect(&m_futureWatcher, SIGNAL(finished()), this, SLOT(loadFilesDone()), Qt::DirectConnection);
 
     // Start the computation.
-    m_futureWatcher.setFuture( QtConcurrent::map(m_tmpdata.begin(), m_tmpdata.end(), LoadFile()) );
+    // m_futureWatcher.setFuture( QtConcurrent::map(m_tmpdata.begin(), m_tmpdata.end(), LoadFile()) );
+    QString error_string;
+    try {
+        std::for_each(m_tmpdata.begin(), m_tmpdata.end(), LoadFile(true));
+    }
+    catch (std::runtime_error &err)
+    {
+        QApplication::restoreOverrideCursor();
+        error_string = QString(err.what());
+        qDebug() << err.what();
+    }
+
+    loadFilesDone(error_string);
 }
 
-void FitsImporter::loadFilesDone()
-{ 
+void FitsImporter::loadFilesDone(QString error_string)
+{
     qDebug() << "Data loaded ... move to internal structure!";
-    disconnect(&m_futureWatcher, SIGNAL(finished()), this, SLOT(loadFilesDone()));
+    //disconnect(&m_futureWatcher, SIGNAL(finished()), this, SLOT(loadFilesDone()));
+
+    m_data.clear();
 
     int idx = 0;
-    BOOST_FOREACH(const HdrCreationItem& i, m_tmpdata) {
-        if ( i.isValid() ) {
-            qDebug() << QString("Insert data for %1").arg(i.filename());
-            m_data.push_back(i);
-            m_previewFrame->getLabel(idx)->setPixmap(QPixmap::fromImage(*(i.qimage())));
+    //BOOST_FOREACH(const HdrCreationItem& i, m_tmpdata)
+    for (HdrCreationItemContainer::iterator i = m_tmpdata.begin(); i != m_tmpdata.end(); ++i)
+    {
+        if ( i->isValid() )
+        {
+            qDebug() << QString("Insert data for %1").arg(i->filename());
+            m_data.push_back(*i);
+            m_previewFrame->getLabel(idx)->setPixmap(QPixmap::fromImage(i->qimage()));
             m_previewFrame->getLabel(idx)->setEnabled(true);
         }
-        else if (i.filename().isEmpty()) {
-            m_data.push_back(i);
-            idx++;
-            continue;
+        else if ( i->filename().isEmpty() )
+        {
+            m_data.push_back(*i);
         }
-        else {
-            QMessageBox::warning(0,"", tr("Cannot load FITS image %1").arg(i.filename()), QMessageBox::Ok, QMessageBox::NoButton);
+        else
+        {
+            QMessageBox::warning(0,"", tr("Cannot load FITS image %1. \nERROR: %2").arg(i->filename()).arg(error_string), QMessageBox::Ok, QMessageBox::NoButton);
             m_data.clear();
             m_tmpdata.clear();
-            m_channels.clear();
             m_contents.clear();
             m_qimages.clear();
-            m_ui->pushButtonLoad->setEnabled(true);
-            QApplication::restoreOverrideCursor();
+            m_Ui->pushButtonLoad->setEnabled(true);
+            //QApplication::restoreOverrideCursor();
             return;
         }
         idx++;
@@ -238,22 +260,21 @@ void FitsImporter::loadFilesDone()
         QMessageBox::warning(0,"", tr("FITS images have different size"), QMessageBox::Ok, QMessageBox::NoButton);
         m_data.clear();
         m_tmpdata.clear();
-        m_channels.clear();
         m_contents.clear();
         m_qimages.clear();
-        m_ui->pushButtonLoad->setEnabled(true);
-        m_ui->pushButtonPreview->setEnabled(true);
+        m_Ui->pushButtonLoad->setEnabled(true);
+        m_Ui->pushButtonPreview->setEnabled(true);
         QApplication::restoreOverrideCursor();
         return;
     }
-    m_ui->pushButtonReset->setEnabled(true);
-    m_ui->pushButtonClockwise->setEnabled(true);
-    m_ui->pushButtonPreview->setEnabled(true);
+    m_Ui->pushButtonReset->setEnabled(true);
+    m_Ui->pushButtonClockwise->setEnabled(true);
+    m_Ui->pushButtonPreview->setEnabled(true);
     m_tmpdata.clear();
     buildContents();
     //buildPreview();
     QApplication::restoreOverrideCursor();
-    m_ui->wizardPageLoadFiles->setCompleteStatus(ExtWizardPage::CompleteStatus::Undefined);
+    m_Ui->wizardPageLoadFiles->setCompleteStatus(ExtWizardPage::CompleteStatus::Undefined);
 }
 
 bool isValid(HdrCreationItem& item)
@@ -263,35 +284,33 @@ bool isValid(HdrCreationItem& item)
 
 void FitsImporter::buildPreview()
 {
-    float redRed = m_ui->dsbRedRed->value();
-    float redGreen = m_ui->dsbRedGreen->value();
-    float redBlue = m_ui->dsbRedBlue->value();
-    float greenRed = m_ui->dsbGreenRed->value();
-    float greenGreen = m_ui->dsbGreenGreen->value();
-    float greenBlue = m_ui->dsbGreenBlue->value();
-    float blueRed = m_ui->dsbBlueRed->value();
-    float blueGreen = m_ui->dsbBlueGreen->value();
-    float blueBlue = m_ui->dsbBlueBlue->value();
-    float gamma = m_ui->vsGamma->value()/10000.0f;
+    float redRed = m_Ui->dsbRedRed->value();
+    float greenGreen = m_Ui->dsbGreenGreen->value();
+    float blueBlue = m_Ui->dsbBlueBlue->value();
+    float gamma = (m_Ui->vsGamma->value()/10000.0f)*3.f;
 
+    Q_ASSERT(gamma >= 0.f);
+    Q_ASSERT(gamma <= 3.f);
+    qDebug() << "Gamma " << gamma;
 
-    QImage tempImage(previewWidth,
-                     previewHeight,
-                     QImage::Format_ARGB32_Premultiplied);
-    
-    if (m_luminosityChannel != "") {
-        for (int j = 0; j < previewHeight; j++) 
+    QImage tempImage(previewWidth, previewHeight, QImage::Format_ARGB32_Premultiplied);
+
+    ConvertSample<float, uint8_t> toFloat;
+    ConvertToQRgb convertToQRgb(1.f + gamma);
+    if (!m_luminosityChannel.isEmpty())
+    {
+        for (int j = 0; j < previewHeight; j++)
         {
-            for (int i = 0; i < previewWidth; i++) 
+            for (int i = 0; i < previewWidth; i++)
             {
-                float red = qRed(m_qimages[0].pixel(i, j))/255.0f;
-                float green = qRed(m_qimages[1].pixel(i, j))/255.0f;
-                float blue = qRed(m_qimages[2].pixel(i, j))/255.0f;
-                float luminance = qRed(m_qimages[3].pixel(i, j))/255.0f;
-                float h_alpha = qRed(m_qimages[4].pixel(i, j))/255.0f;
-                float r = redRed * red + redGreen * green + redBlue * blue;
-                float g = greenRed * red + greenGreen * green + greenBlue * blue;
-                float b = blueRed * red + blueGreen * green + blueBlue * blue;
+                float red = toFloat(qRed(m_qimages[0].pixel(i, j)));
+                float green = toFloat(qRed(m_qimages[1].pixel(i, j)));
+                float blue = toFloat(qRed(m_qimages[2].pixel(i, j)));
+                float luminance = toFloat(qRed(m_qimages[3].pixel(i, j)));
+                float h_alpha = toFloat(qRed(m_qimages[4].pixel(i, j)));
+                float r = redRed * red /*+ redGreen * green + redBlue * blue*/;
+                float g = /*greenRed * red +*/ greenGreen * green /*+ greenBlue * blue*/;
+                float b = /*blueRed * red + blueGreen * green + */blueBlue * blue;
                 float h, s, l;
                 rgb2hsl(r, g, b, h, s, l);
                 hsl2rgb(h, s, luminance, r, g, b);
@@ -305,24 +324,24 @@ void FitsImporter::buildPreview()
                 if (redH > 1.0f)
                     redH = 1.0f;
                 QRgb rgb;
-                ConvertToQRgb convert(gamma);
-                convert(redH, g, b, rgb); 
+                convertToQRgb(redH, g, b, rgb);
                 tempImage.setPixel(i, j, rgb);
             }
-        } 
+        }
     }
-    else {
-        for (int j = 0; j < previewHeight; j++) 
+    else
+    {
+        for (int j = 0; j < previewHeight; j++)
         {
-            for (int i = 0; i < previewWidth; i++) 
+            for (int i = 0; i < previewWidth; i++)
             {
-                float red = qRed(m_qimages[0].pixel(i, j))/255.0f;
-                float green = qRed(m_qimages[1].pixel(i, j))/255.0f;
-                float blue = qRed(m_qimages[2].pixel(i, j))/255.0f;
-                float h_alpha = qRed(m_qimages[4].pixel(i, j))/255.0f;
-                float r = redRed * red + redGreen * green + redBlue * blue;
-                float g = greenRed * red + greenGreen * green + greenBlue * blue;
-                float b = blueRed * red + blueGreen * green + blueBlue * blue;
+                float red = toFloat(qRed(m_qimages[0].pixel(i, j)));
+                float green = toFloat(qRed(m_qimages[1].pixel(i, j)));
+                float blue = toFloat(qRed(m_qimages[2].pixel(i, j)));
+                float h_alpha = toFloat(qRed(m_qimages[4].pixel(i, j)));
+                float r = redRed * red /*+ redGreen * green + redBlue * blue*/;
+                float g = /*greenRed * red +*/ greenGreen * green /*+ greenBlue * blue*/;
+                float b = /*blueRed * red + blueGreen * green +*/ blueBlue * blue;
                 float redH = r + 0.2f * h_alpha;
                 if (r > 1.0f)
                     r = 1.0f;
@@ -332,95 +351,117 @@ void FitsImporter::buildPreview()
                     b = 1.0f;
                 if (redH > 1.0f)
                     redH = 1.0f;
+
                 QRgb rgb;
-                ConvertToQRgb convert(gamma);
-                convert(redH, g, b, rgb); 
+                convertToQRgb(redH, g, b, rgb);
                 tempImage.setPixel(i, j, rgb);
             }
-        } 
-    }    
-    m_ui->previewLabel->setPixmap(QPixmap::fromImage(tempImage));
+        }
+    }
+    m_Ui->previewLabel->setPixmap(QPixmap::fromImage(tempImage));
 }
 
 void FitsImporter::buildContents()
 {
-    HdrCreationItemContainer::iterator it;
-    it = std::find_if(m_data.begin(), m_data.end(), isValid);
+    HdrCreationItemContainer::iterator it =
+            std::find_if(m_data.begin(), m_data.end(), isValid);
+
+    if (it == m_data.end())
+        return;
+
     m_width = it->frame()->getWidth();
     m_height = it->frame()->getHeight();
 
-    for (size_t i = 0; i < m_data.size(); i++) {
+    for (size_t i = 0; i < m_data.size(); i++)
+    {
          m_contents.push_back(std::vector<float>(m_width*m_height));
     }
 
-    for (size_t i = 0; i < m_data.size(); i++) {
-        if (m_data[i].filename().isEmpty()) {
+    float datamax = std::numeric_limits<float>::min();
+    float datamin = std::numeric_limits<float>::max();
+    for (size_t i = 0; i < m_data.size(); i++)
+    {
+        if (!m_data[i].filename().isEmpty())
+        {
+            datamin = std::min(datamin, m_data[i].getMin());
+            datamax = std::max(datamax, m_data[i].getMax());
+        }
+    }
+#ifndef NDEBUG
+        std::cout << "FitsImporter datamin = " << datamin << std::endl;
+        std::cout << "FitsImporter datamax = " << datamax << std::endl;
+#endif
+
+    for (size_t i = 0; i < m_data.size(); i++)
+    {
+        if (m_data[i].filename().isEmpty())
+        {
             std::fill(m_contents[i].begin(), m_contents[i].end(), 0.0f);
             QImage tmpImage(previewWidth, previewHeight, QImage::Format_ARGB32_Premultiplied);
             tmpImage.fill(0);
             m_qimages.push_back(tmpImage);
         }
-        else {
+        else
+        {
             Channel *C = m_data[i].frame()->getChannel("X");
-            std::copy(C->begin(), C->end(), m_contents[i].begin()); 
-            m_qimages.push_back(m_data[i].qimage()->scaled(previewWidth, previewHeight));
-            
+            pfs::colorspace::Normalizer normalize(datamin, datamax);
+
+            std::transform(C->begin(), C->end(), C->begin(), normalize);
+            std::copy(C->begin(), C->end(), m_contents[i].begin());
+            m_qimages.push_back(m_data[i].qimage().scaled(previewWidth, previewHeight));
         }
     }
 }
 
 void FitsImporter::buildFrame()
 {
-    float redRed = m_ui->dsbRedRed->value();
-    float redGreen = m_ui->dsbRedGreen->value();
-    float redBlue = m_ui->dsbRedBlue->value();
-    float greenRed = m_ui->dsbGreenRed->value();
-    float greenGreen = m_ui->dsbGreenGreen->value();
-    float greenBlue = m_ui->dsbGreenBlue->value();
-    float blueRed = m_ui->dsbBlueRed->value();
-    float blueGreen = m_ui->dsbBlueGreen->value();
-    float blueBlue = m_ui->dsbBlueBlue->value();
+    float redRed = m_Ui->dsbRedRed->value();
+    float greenGreen = m_Ui->dsbGreenGreen->value();
+    float blueBlue = m_Ui->dsbBlueBlue->value();
 
     m_frame = new Frame(m_width, m_height);
     Channel *Xc, *Yc, *Zc;
     m_frame->createXYZChannels(Xc, Yc, Zc);
 
-    if (m_luminosityChannel != "") {
-        for (size_t i = 0; i < m_width*m_height; i++) 
+    if (!m_luminosityChannel.isEmpty())
+    {
+        for (size_t i = 0; i < m_width*m_height; i++)
         {
-            float r = redRed * m_contents[0][i] + redGreen * m_contents[1][i] + redBlue * m_contents[2][i];
-            float g = greenRed * m_contents[0][i] + greenGreen * m_contents[1][i] + greenBlue * m_contents[2][i];
-            float b = blueRed * m_contents[0][i] + blueGreen * m_contents[1][i] + blueBlue * m_contents[2][i];
+            //float r = redRed * m_contents[2][i];
+            float r = redRed * m_contents[0][i];
+            float g = greenGreen * m_contents[1][i];
+            float b = blueBlue * m_contents[2][i];
             float h, s, l;
             rgb2hsl(r, g, b, h, s, l);
             hsl2rgb(h, s, m_contents[3][i], r, g, b);
             (*Xc)(i) = r + m_contents[4][i];
             (*Yc)(i) = g;
             (*Zc)(i) = b;
-        } 
+        }
     }
-    else {
-        for (size_t i = 0; i < m_width*m_height; i++) 
+    else
+    {
+        for (size_t i = 0; i < m_width*m_height; i++)
         {
-            float r = redRed * m_contents[0][i] + redGreen * m_contents[1][i] + redBlue * m_contents[2][i];
-            float g = greenRed * m_contents[0][i] + greenGreen * m_contents[1][i] + greenBlue * m_contents[2][i];
-            float b = blueRed * m_contents[0][i] + blueGreen * m_contents[1][i] + blueBlue * m_contents[2][i];
+            float r = redRed * m_contents[0][i];
+            float g = greenGreen * m_contents[1][i];
+            float b = blueBlue * m_contents[2][i];
             (*Xc)(i) = r + m_contents[4][i];
             (*Yc)(i) = g;
             (*Zc)(i) = b;
-        } 
-    }    
+        }
+    }
 }
 
 void FitsImporter::align_with_ais()
 {
     m_contents.clear();
-    m_align = new Align(&m_data, false, 2, 0.0f, 65535.0f); 
-    connect(m_align, SIGNAL(finishedAligning(int)), this, SLOT(ais_finished(int)));
-    connect(m_align, SIGNAL(failedAligning(QProcess::ProcessError)), this, SLOT(ais_failed_slot(QProcess::ProcessError)));
-    connect(m_align, SIGNAL(dataReady(QByteArray)), this, SLOT(readData(QByteArray)));
-  
-    m_align->align_with_ais(m_ui->autoCropCheckBox->isChecked());
+    m_align.reset(new Align(m_data, false, 2, 0.0f, 65535.0f));
+    connect(m_align.get(), SIGNAL(finishedAligning(int)), this, SLOT(ais_finished(int)));
+    connect(m_align.get(), SIGNAL(failedAligning(QProcess::ProcessError)), this, SLOT(ais_failed_slot(QProcess::ProcessError)));
+    connect(m_align.get(), SIGNAL(dataReady(QByteArray)), this, SLOT(readData(QByteArray)));
+
+    m_align->align_with_ais(m_Ui->autoCropCheckBox->isChecked());
 }
 
 void FitsImporter::ais_finished(int exitcode)
@@ -437,7 +478,7 @@ void FitsImporter::ais_finished(int exitcode)
         buildContents();
         //buildFrame();
     }
-    m_ui->wizardPageAlignment->setCompleteStatus(ExtWizardPage::CompleteStatus::Undefined);
+    m_Ui->wizardPageAlignment->setCompleteStatus(ExtWizardPage::CompleteStatus::Undefined);
 }
 
 void FitsImporter::ais_failed_slot(QProcess::ProcessError error)
@@ -452,14 +493,16 @@ bool FitsImporter::framesHaveSameSize()
 {
     HdrCreationItemContainer::iterator it;
     it = std::find_if(m_data.begin(), m_data.end(), isValid);
+    if (it == m_data.end())
+        return false;
     const size_t width = it->frame()->getWidth();
     const size_t height = it->frame()->getHeight();
-    for ( HdrCreationItemContainer::const_iterator it = m_data.begin() + 1, 
+    for ( HdrCreationItemContainer::const_iterator it = m_data.begin() + 1,
           itEnd = m_data.end(); it != itEnd; ++it) {
         if (it->filename().isEmpty())
             continue;
         if (it->frame()->getWidth() != width || it->frame()->getHeight() != height)
-            return false; 
+            return false;
     }
     return true;
 }
@@ -474,7 +517,7 @@ void FitsImporter::readData(QByteArray data)
     if (data.contains(QChar(0x01B).toLatin1()))
         data.replace(QChar(0x01B).toLatin1(), "");
 
-    m_ui->textEdit->append(data);
+    m_Ui->textEdit->append(data);
     if (data.contains(": remapping")) {
         QRegExp exp("\\:\\s*(\\d+)\\s*");
         exp.indexIn(QString(data.data()));
@@ -504,34 +547,34 @@ void FitsImporter::align_with_mtb()
 void FitsImporter::on_pushButtonClockwise_clicked()
 {
     QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
-    m_ui->pushButtonClockwise->setEnabled(false);
+    m_Ui->pushButtonClockwise->setEnabled(false);
     int index = m_previewFrame->getSelectedLabel();
 
-    boost::scoped_ptr<Frame> rotatedHalf(pfs::rotate(m_data[index].frame().get(), true));
-    boost::scoped_ptr<Frame> rotated(pfs::rotate(rotatedHalf.get(), true));
+    std::unique_ptr<Frame> rotatedHalf(pfs::rotate(m_data[index].frame().get(), true));
+    std::unique_ptr<Frame> rotated(pfs::rotate(rotatedHalf.get(), true));
     m_data[index].frame()->swap(*rotated);
     rotatedHalf.reset();
     rotated.reset();
 
     RefreshPreview refresh;
     refresh(m_data[index]);
-    m_previewFrame->getLabel(index)->setPixmap(QPixmap::fromImage(*(m_data[index].qimage())));
-    if (m_ui->pushButtonPreview->isChecked())
+    m_previewFrame->getLabel(index)->setPixmap(QPixmap::fromImage(m_data[index].qimage()));
+    if (m_Ui->pushButtonPreview->isChecked())
     {
         m_previewLabel->setPixmap(*m_previewFrame->getLabel(m_previewFrame->getSelectedLabel())->pixmap());
     }
     Channel *C = m_data[index].frame()->getChannel("X");
-    std::copy(C->begin(), C->end(), m_contents[index].begin()); 
-    QImage tmp = m_data[index].qimage()->scaled(previewWidth, previewHeight);
+    std::copy(C->begin(), C->end(), m_contents[index].begin());
+    QImage tmp = m_data[index].qimage().scaled(previewWidth, previewHeight);
     m_qimages[index].swap(tmp);
     //buildPreview();
-    m_ui->pushButtonClockwise->setEnabled(true);
+    m_Ui->pushButtonClockwise->setEnabled(true);
     QApplication::restoreOverrideCursor();
 }
 
 void FitsImporter::on_pushButtonPreview_clicked()
 {
-    if (m_ui->pushButtonPreview->isChecked()) {
+    if (m_Ui->pushButtonPreview->isChecked()) {
         m_previewLabel->setPixmap(*m_previewFrame->getLabel(m_previewFrame->getSelectedLabel())->pixmap());
         m_previewLabel->show();
     }
@@ -541,7 +584,7 @@ void FitsImporter::on_pushButtonPreview_clicked()
 
 void FitsImporter::previewLabelSelected(int index)
 {
-    if (m_ui->pushButtonPreview->isChecked()) {
+    if (m_Ui->pushButtonPreview->isChecked()) {
         m_previewLabel->setPixmap(*m_previewFrame->getLabel(m_previewFrame->getSelectedLabel())->pixmap());
     }
 }
@@ -549,153 +592,51 @@ void FitsImporter::previewLabelSelected(int index)
 void FitsImporter::on_hsRedRed_valueChanged(int newValue)
 {
     float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbRedRed->blockSignals(true);
-    m_ui->dsbRedRed->setValue( value );
-    m_ui->dsbRedRed->blockSignals(oldState);
+    bool oldState = m_Ui->dsbRedRed->blockSignals(true);
+    m_Ui->dsbRedRed->setValue( value );
+    m_Ui->dsbRedRed->blockSignals(oldState);
     buildPreview();
 }
 
 void FitsImporter::on_dsbRedRed_valueChanged(double newValue)
 {
-    bool oldState = m_ui->hsRedRed->blockSignals(true);
-    m_ui->hsRedRed->setValue( (int)(newValue*10000) );
-    m_ui->hsRedRed->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_hsRedGreen_valueChanged(int newValue)
-{
-    float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbRedGreen->blockSignals(true);
-    m_ui->dsbRedGreen->setValue( value );
-    m_ui->dsbRedGreen->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_dsbRedGreen_valueChanged(double newValue)
-{
-    bool oldState = m_ui->hsRedGreen->blockSignals(true);
-    m_ui->hsRedGreen->setValue( (int)(newValue*10000) );
-    m_ui->hsRedGreen->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_hsRedBlue_valueChanged(int newValue)
-{
-    float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbRedBlue->blockSignals(true);
-    m_ui->dsbRedBlue->setValue( value );
-    m_ui->dsbRedBlue->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_dsbRedBlue_valueChanged(double newValue)
-{
-    bool oldState = m_ui->hsRedBlue->blockSignals(true);
-    m_ui->hsRedBlue->setValue( (int)(newValue*10000) );
-    m_ui->hsRedBlue->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_hsGreenRed_valueChanged(int newValue)
-{
-    float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbGreenRed->blockSignals(true);
-    m_ui->dsbGreenRed->setValue( value );
-    m_ui->dsbGreenRed->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_dsbGreenRed_valueChanged(double newValue)
-{
-    bool oldState = m_ui->hsGreenRed->blockSignals(true);
-    m_ui->hsGreenRed->setValue( (int)(newValue*10000) );
-    m_ui->hsGreenRed->blockSignals(oldState);
+    bool oldState = m_Ui->hsRedRed->blockSignals(true);
+    m_Ui->hsRedRed->setValue( (int)(newValue*10000) );
+    m_Ui->hsRedRed->blockSignals(oldState);
     buildPreview();
 }
 
 void FitsImporter::on_hsGreenGreen_valueChanged(int newValue)
 {
     float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbGreenGreen->blockSignals(true);
-    m_ui->dsbGreenGreen->setValue( value );
-    m_ui->dsbGreenGreen->blockSignals(oldState);
+    bool oldState = m_Ui->dsbGreenGreen->blockSignals(true);
+    m_Ui->dsbGreenGreen->setValue( value );
+    m_Ui->dsbGreenGreen->blockSignals(oldState);
     buildPreview();
 }
 
 void FitsImporter::on_dsbGreenGreen_valueChanged(double newValue)
 {
-    bool oldState = m_ui->hsGreenGreen->blockSignals(true);
-    m_ui->hsGreenGreen->setValue( (int)(newValue*10000) );
-    m_ui->hsGreenGreen->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_hsGreenBlue_valueChanged(int newValue)
-{
-    float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbGreenBlue->blockSignals(true);
-    m_ui->dsbGreenBlue->setValue( value );
-    m_ui->dsbGreenBlue->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_dsbGreenBlue_valueChanged(double newValue)
-{
-    bool oldState = m_ui->hsGreenBlue->blockSignals(true);
-    m_ui->hsGreenBlue->setValue( (int)(newValue*10000) );
-    m_ui->hsGreenBlue->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_hsBlueRed_valueChanged(int newValue)
-{
-    float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbBlueRed->blockSignals(true);
-    m_ui->dsbBlueRed->setValue( value );
-    m_ui->dsbBlueRed->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_dsbBlueRed_valueChanged(double newValue)
-{
-    bool oldState = m_ui->hsBlueRed->blockSignals(true);
-    m_ui->hsBlueRed->setValue( (int)(newValue*10000) );
-    m_ui->hsBlueRed->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_hsBlueGreen_valueChanged(int newValue)
-{
-    float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbBlueGreen->blockSignals(true);
-    m_ui->dsbBlueGreen->setValue( value );
-    m_ui->dsbBlueGreen->blockSignals(oldState);
-    buildPreview();
-}
-
-void FitsImporter::on_dsbBlueGreen_valueChanged(double newValue)
-{
-    bool oldState = m_ui->hsBlueGreen->blockSignals(true);
-    m_ui->hsBlueGreen->setValue( (int)(newValue*10000) );
-    m_ui->hsBlueGreen->blockSignals(oldState);
+    bool oldState = m_Ui->hsGreenGreen->blockSignals(true);
+    m_Ui->hsGreenGreen->setValue( (int)(newValue*10000) );
+    m_Ui->hsGreenGreen->blockSignals(oldState);
     buildPreview();
 }
 
 void FitsImporter::on_hsBlueBlue_valueChanged(int newValue)
 {
     float value = ((float)newValue)/10000.f;
-    bool oldState = m_ui->dsbBlueBlue->blockSignals(true);
-    m_ui->dsbBlueBlue->setValue( value );
-    m_ui->dsbBlueBlue->blockSignals(oldState);
+    bool oldState = m_Ui->dsbBlueBlue->blockSignals(true);
+    m_Ui->dsbBlueBlue->setValue( value );
+    m_Ui->dsbBlueBlue->blockSignals(oldState);
     buildPreview();
 }
 
 void FitsImporter::on_dsbBlueBlue_valueChanged(double newValue)
 {
-    bool oldState = m_ui->hsBlueBlue->blockSignals(true);
-    m_ui->hsBlueBlue->setValue( (int)(newValue*10000) );
-    m_ui->hsBlueBlue->blockSignals(oldState);
+    bool oldState = m_Ui->hsBlueBlue->blockSignals(true);
+    m_Ui->hsBlueBlue->setValue( (int)(newValue*10000) );
+    m_Ui->hsBlueBlue->blockSignals(oldState);
     buildPreview();
 }
 
@@ -708,15 +649,14 @@ void FitsImporter::on_pushButtonReset_clicked()
 {
     m_data.clear();
     m_tmpdata.clear();
-    m_channels.clear();
     m_contents.clear();
     m_qimages.clear();
-    m_ui->lineEditRed->clear();
-    m_ui->lineEditGreen->clear();
-    m_ui->lineEditBlue->clear();
-    m_ui->lineEditLuminosity->clear();
-    m_ui->lineEditH->clear();
-    m_ui->previewLabel->clear();
+    m_Ui->lineEditRed->clear();
+    m_Ui->lineEditGreen->clear();
+    m_Ui->lineEditBlue->clear();
+    m_Ui->lineEditLuminosity->clear();
+    m_Ui->lineEditH->clear();
+    m_Ui->previewLabel->clear();
     m_previewFrame->getLabel(0)->clear();
     m_previewFrame->getLabel(1)->clear();
     m_previewFrame->getLabel(2)->clear();
@@ -727,8 +667,8 @@ void FitsImporter::on_pushButtonReset_clicked()
     m_previewFrame->getLabel(2)->setEnabled(false);
     m_previewFrame->getLabel(3)->setEnabled(false);
     m_previewFrame->getLabel(4)->setEnabled(false);
-    m_ui->pushButtonPreview->setEnabled(false);
-    m_ui->pushButtonClockwise->setEnabled(false);
+    m_Ui->pushButtonPreview->setEnabled(false);
+    m_Ui->pushButtonClockwise->setEnabled(false);
 }
 
 int FitsImporter::nextId() const
@@ -736,7 +676,7 @@ int FitsImporter::nextId() const
     int current = currentId();
     if (current == 21) // load files
     {
-        return  m_ui->alignCheckBox->isChecked()
+        return  m_Ui->alignCheckBox->isChecked()
             ? 22  // alignment progress
             : 99; // preview page
     }
@@ -753,8 +693,8 @@ void FitsImporter::initializePage(int id)
 {
     int current = currentId();
     if (current == 22)  // alignment progress
-    { 
-        if (m_ui->ais_radioButton->isChecked()) {
+    {
+        if (m_Ui->ais_radioButton->isChecked()) {
             align_with_ais();
         }
         else
